#include <glib-unix.h>
#include <assert.h>
#include "shell.h"
#include "background.h"
#include "launcher.h"
#include "wayland.h"
#include "foreign.h"
#include "session.h"

static gchar **g_shell_argv;

static void activate(GtkApplication *app, void *data)
{
    (void)app;
    (void)data;

    DiyaShell *shell = data;
    diya_shell_lock(shell);
}

static gboolean restart(gpointer d)
{
    (void)d;
    gint i, fdlimit;

    fdlimit = (int)sysconf(_SC_OPEN_MAX);
    g_debug("reload: closing fd's %d to %d", STDERR_FILENO + 1, fdlimit);
    for (i = STDERR_FILENO + 1; i < fdlimit; i++)
    {
        fcntl(i, F_SETFD, FD_CLOEXEC);
    }
    g_debug("reload: exec: %s", g_shell_argv[0]);
    execvp(g_shell_argv[0], g_shell_argv);
    exit(1);
    return FALSE;
}

static void startup(GtkApplication *app, void *data)
{
    DiyaShell *shell = data;

    diya_shell_launcher_init(shell);
    diya_shell_wayland_init(shell);
    diya_shell_init_background(shell);

    GtkCssProvider *provider = gtk_css_provider_new();
    const char *css = "#launcher {background-color: red;} #background {background-image:url(\"file:///etc/xdg/labwc/wpp.jpg\");background-size: cover;}";
    gtk_css_provider_load_from_string(provider, css);
    gtk_style_context_add_provider_for_display(
        gdk_display_get_default(), GTK_STYLE_PROVIDER(provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
    // g_signal_connect (win, "destroy", G_CALLBACK (before_destroy), provider);
    g_object_unref(provider);
    // CSS support
    // set color for it
    // g_timeout_add (100,(GSourceFunc )shell_timer,NULL);
    g_unix_signal_add(SIGHUP, (GSourceFunc)restart, NULL);
    g_unix_signal_add(SIGINT,(GSourceFunc)g_application_quit,(void*)app);
}

static void session_locked(DiyaShell* shell, void* data)
{
    (void)data;
    assert(shell);
    g_warning("session_locked callback triggered");
}

static void session_unlocked(DiyaShell* shell, void* data)
{
    (void)data;
    assert(shell);
    g_warning("session_unlocked callback triggered");
}

int main(int argc, char *argv[])
{
    g_shell_argv = g_malloc0(sizeof(gchar *) * (argc + 1));
    for (int i = 0; i < argc; i++)
    {
        g_shell_argv[i] = argv[i];
    }
    GtkApplication *app = gtk_application_new("dev.iohub.diya.shell", G_APPLICATION_DEFAULT_FLAGS);
    DiyaShell *shell = diya_shell_new(app);
    g_signal_connect(app, "startup", G_CALLBACK(startup), (void *)shell);
    g_signal_connect(app, "activate", G_CALLBACK(activate), (void *)shell);
    g_signal_connect(shell, "session-locked", G_CALLBACK(session_locked), NULL);
    g_signal_connect(shell, "session-unlocked", G_CALLBACK(session_unlocked), NULL);
    // g_signal_connect(app, "shutdown", G_CALLBACK(shutdown), (void*)shell);
    int status = g_application_run(G_APPLICATION(app), argc, argv);
    g_warning("Exiting SHELL");
    g_object_unref(app);
    g_object_unref(shell);
    return status;
}
